//--------------------------------------------------------------------------------------------------
//
// Title       : alu_controls
// Design      : lib378
// Author      : Steven
// Company     : UW CSE
//
//-------------------------------------------------------------------------------------------------
//
// File        : alu_controls.v
// Generated   : Tue Aug  29 15:39:43 2006
// From        : interface description file
// By          : Itf2Vhdl ver. 1.20
//
//-------------------------------------------------------------------------------------------------
//
// Description :  ALU Control unit that generates the specific control signals for the ALU based on\
//                input functions and control signals
//
//-------------------------------------------------------------------------------------------------
`timescale 1 ns / 1 ps

module ALUControl ( ALUOp, Func, Opcode, ALUCtl) /* synthesis syn_black_box */ ;
	
	input [1:0] ALUOp ;
	input [5:0] Func ;
	input [5:0] Opcode ;
	output [4:0] ALUCtl;
	
	// synopsys translate_off
	
	wire [1:0] ALUOp ;
	wire [5:0] Func ;
	wire [5:0] Opcode ;
	wire [4:0] ALUCtl ;
	
	
	
	//}} End of automatically maintained section
	
	// -- Enter your statements here -- //
	
	localparam SLL		= 6'b000000; // funct = 0
	localparam SRL		= 6'b000010; // funct = 2
	localparam SRA		= 6'b000011; // funct = 3
	localparam SLLV		= 6'b000100; // funct = 4
	localparam SRLV		= 6'b000110; // funct = 6
	localparam SRAV		= 6'b000111; // funct = 7
	localparam JR		= 6'b001000; // funct = 8
	localparam JALR		= 6'b001001; // funct = 9
	localparam ADD		= 6'b100000; // funct = 32
	localparam ADDU		= 6'b100001; // funct = 33
	localparam SUB		= 6'b100010; // funct = 34
	localparam SUBU		= 6'b100011; // funct = 35
	localparam AND		= 6'b100100; // funct = 36
	localparam OR		= 6'b100101; // funct = 37
	localparam XOR		= 6'b100110; // funct = 38
	localparam NOR		= 6'b100111; // funct = 39
	localparam SLT		= 6'b101010; // funct = 42
	localparam SLTU		= 6'b101011; // funct = 43
	localparam MULT		= 6'b010100; //funct = 24
	localparam MULTU	= 6'b011001; //funct = 25	  
	localparam MFHI		= 6'b010000; //funct = 16
	localparam MTHI		= 6'b010001; //funct = 17
	localparam MFLO		= 6'b010010; //funct = 18
	localparam MTLO		= 6'b010011; //funct = 19
	//
	localparam ADDI 	= 6'b001000;
	localparam ADDIU 	= 6'b001001;
	localparam SLTI		= 6'b001010;
	localparam SLTIU	= 6'b001011;
	localparam ANDI		= 6'b001100;
	localparam ORI		= 6'b001101;
	localparam XORI		= 6'b001110;
	localparam LUI		= 6'b001111;
	
	/* choose whether Func or Opcode determines the operation */
	wire [5:0] Function = (ALUOp[0])? Opcode : Func;
	/* Define signals for each of the modes */
	wire I_format = ALUOp[1]&ALUOp[0];
	wire R_format = ALUOp[1]&~ALUOp[0];
	wire Force_Sub = ~ALUOp[1]&ALUOp[0];
	wire Force_Add = ~ALUOp[1]&~ALUOp[0];

	// Operation is the same for SLL and SLLV, only active when ALUOp is 10
	wire shift_ll =  R_format &((Function === SLL) | (Function === SLLV));
	wire shift_rl =  R_format &((Function === SRL) | (Function === SRLV));
	wire shift_ra =  R_format &((Function === SRA) | (Function === SRAV));	
	wire shift_op = (shift_ll | shift_rl | shift_ra);

	// OR is an OR, and JR doesn't care, so set it to OR
	wire r_or_op = (R_format & ((Function === OR) | (Function === JR)));
	// ORI is an OR, and LUI is done in the extender, so just OR it with 0
	wire i_or_op = (I_format & ((Function === ORI)|(Function === LUI)));
	// Combine the two
	wire or_op = (r_or_op | i_or_op);

	// XOR or XORI 
	wire xor_op		= (R_format & (Function === XOR))  | (I_format & (Function === XORI)); 
	wire slt_op		= (R_format & (Function === SLT))  | (I_format & (Function === SLTI));
	wire sltu_op	= (R_format & (Function === SLTU)) | (I_format & (Function === SLTIU));
	wire add_op		= (R_format & (Function === ADD))  | (I_format & (Function === ADDI)) | Force_Add;
	// Force JALR to perform an ADDU 
	wire addu_op	= (R_format & ((Function === ADDU)|(Function === JALR))) | (I_format & (Function === ADDIU));
	wire nor_op		= (R_format & (Function === NOR)); 
	wire sub_op		= (R_format & (Function === SUB)) | Force_Sub; // if ALUOp = 01, make it subtract
	wire subu_op	= (R_format & (Function === SUBU));
	
	wire mult_op	= R_format & (Function === MULT);
	wire multu_op	= R_format & (Function === MULTU);
	wire mfhi_op	= R_format & (Function === MFHI);
	wire mthi_op	= R_format & (Function === MTHI);
	wire mflo_op	= R_format & (Function === MFLO);
	wire mtlo_op	= R_format & (Function === MTLO);
	
//  Mapping of Control signals to ALU operations
//  AND   = 0,  00000
//  OR    = 1,  00001
//  ADD   = 2,  00010
//  ADDU  = 3,  00011
//  SUBU  = 4,  00100
//  SUB   = 6,  00110
//  SLT   = 7,  00111
//  SLTU  = 8,  01000
//  XOR	  = 9,  01001
//  SLL   = 10,	01010
//  SRL   = 11,	01011
//  NOR	  = 12,	01100
//  SRA	  = 13,	01101
//	MULT  = 14, 01110
//	MULTU = 15,	01111
//	MFHI  = 16,	10000
//	MTHI  = 17,	10001
//	MFLO  = 18,	10010
//	MTLO  = 19;	10011

	// As seen above, ALUCtl[4] is only set for MTXX,MFXX
	assign ALUCtl[4] = ((mfhi_op)|(mthi_op)|(mflo_op)|(mtlo_op));
	
	assign ALUCtl[3] = ((shift_op)|(xor_op)|(nor_op)|(sltu_op)|(mult_op)|(multu_op));
	// sub_op considers case where ALUOp = 01 and we force a subtraction
	assign ALUCtl[2] = ((shift_ra)|(sub_op)|(subu_op)|(nor_op)|(slt_op)|(mult_op)|(multu_op));
	
	assign ALUCtl[1] = ((shift_rl)|(shift_ll)|(addu_op)|(add_op)|(sub_op)|(slt_op)|(mult_op)|(multu_op)|(mflo_op)|(mtlo_op));
	assign ALUCtl[0] = ((shift_rl)|(shift_ra)|(or_op)|(addu_op)|(xor_op)|(slt_op)|(mthi_op)|(multu_op)|(mtlo_op)); 

	// synopsys translate_on
	
endmodule
