//--------------------------------------------------------------------------------------------------
//
// Title       : arithmetic modules
// Design      : Base Components
// Author      : Steven
// Company     : UW CSE
//
//-------------------------------------------------------------------------------------------------
//
// File        : arithmetic.v
// Generated   : Thu May 4 20:45:40 2006
// From        : interface description file
// By          : Itf2Vhdl ver. 1.20
//
//-------------------------------------------------------------------------------------------------
//
// Description : 
//
//-------------------------------------------------------------------------------------------------
`timescale 1 ns / 1 ps

module adder ( Out ,InA ,InB ) /* synthesis syn_black_box */ ;
	
	parameter WIDTH = 32;
	
output [WIDTH-1:0] Out;
wire [WIDTH-1:0] Out;
input [WIDTH-1:0] InA ;
wire [WIDTH-1:0] InA ;
input [WIDTH-1:0] InB ;
wire [WIDTH-1:0] InB ; 

//synopsys translate_off

//}} End of automatically maintained section
assign Out = InA + InB;
// -- Enter your statements here -- //


//synopsys translate_on

endmodule

module signed_mult32(A,B,O) /* synthesis syn_black_box */  ;
	input signed [31:0] A,B;
	output signed [63:0] O;
	
	//synopsys translate_off
	
	assign O = A * B;
	
	//synopsys translate_on
endmodule

module unsigned_mult32(A,B,O) /* synthesis syn_black_box */ ;
	input wire [31:0] A,B;
	output wire [63:0] O;
	
	//synopsys translate_off
	assign O = A * B;		
	//synopsys translate_on
endmodule

// Better ALU supports SRA,SRL,SLL,

module MIPSALU ( ALUctl, A, B, ALUOut,Zero,Ovfl, CLK)  /* synthesis syn_black_box */  ;
	input  [31:0] A,B ;
	input  [4:0] ALUctl;
	input  CLK;		
	output wire Zero,Ovfl;
	output reg [31:0] ALUOut ;
	
	//synopsys translate_off
	
	wire signedOp,sameSign,diffOutSign;
	wire [31:0] SHFT_IN,SHFT_OUT,ARITH_OUT;
    wire [4:0] ShftAmt;

	//}} End of automatically maintained section
	
	// -- Enter your statements here -- //
	parameter AND = 0,
	OR = 1,
	ADD = 2,
	ADDU = 3,
	SUBU = 4,
	SUB = 6,
	SLT = 7,
	SLTU = 8,
	XOR	 = 9,
	SLL  = 10,
	SRL  = 11,
	NOR  = 12,
	SRA  = 13,
	MULT = 14,
	MULTU = 15,
	MFHI = 16,
	MTHI = 17,
	MFLO = 18,
	MTLO = 19;
	
	wire [31:0] REV_B,REV_OUT,REV_SHFT_OUT;
	
	bit_reverse in_b (.in(B),.out(REV_B));
	bit_reverse out_b (.in(REV_SHFT_OUT),.out(REV_OUT));
	assign REV_SHFT_OUT = (ALUctl[3:0] == SRA)? ARITH_OUT : SHFT_OUT;
	assign SHFT_IN[31:0] = (ALUctl[3:0] == SLL)? B[31:0] : REV_B;

	barrel32 Shifter(.DIN(SHFT_IN), .DOUT(SHFT_OUT),.SHIFT(ShftAmt));
	bin2unary arith(.in(SHFT_OUT),.out(ARITH_OUT),.bin(ShftAmt),.sign(((ALUctl == SRA) & B[31])));
	
	reg [63:0] hiloreg;	 
	wire [63:0] MultOut,MultuOut,Multiply; 
	
	mVw_2_1_top #(.width(64)) mul_sel(.in0(MultOut),.in1(MultuOut),.sel(ALUctl == MULTU),.out(Multiply));
	
	signed_mult32 smult(.A(A),.B(B),.O(MultOut));
	unsigned_mult32 umult(.A(A),.B(B),.O(MultuOut));
	
	assign Zero = (ALUOut==0); //Zero is true if ALUOut is 0
	assign ShftAmt = A[4:0];	
	
	wire HIwe = (ALUctl == MULT)|(ALUctl == MULTU)|(ALUctl == MTHI);
	wire LOwe = (ALUctl == MULT)|(ALUctl == MULTU)|(ALUctl == MTLO);
	
	
	always @(posedge CLK) begin
		if(HIwe) begin
			hiloreg[63:32] <= (ALUctl == MTHI)? A[31:0] : Multiply[63:32];
		end
		if(LOwe) begin
			hiloreg[31:0] <= (ALUctl == MTLO)? A[31:0] : Multiply[31:0];
		end
	end
	
	always @(ALUctl, A, B,REV_OUT,SHFT_OUT, ShftAmt, hiloreg) begin //reevaluate if these change
			case (ALUctl) /* synthesis parallel_case */
				AND: ALUOut <= A & B;
				OR: ALUOut <= A | B;
				ADD,ADDU: ALUOut <= A + B;  // add / addi, addu / addiu
				SUBU,SUB: ALUOut <= A + (~B) + 1;  // subu, sub
				SLT: ALUOut <= (A[31]^B[31])? (A[31]) : (A < B);	// used by set on less than (SLT / SLTI) 
				SLTU: ALUOut <= A < B ? 1 : 0; // used by SLTU / SLTIU 
				XOR: ALUOut <= A ^ B; // xor / xori 
				NOR: ALUOut <= ~(A | B); // result is nor
				SRL,SRA: ALUOut <= (ShftAmt)? REV_OUT : B;
				SLL: ALUOut <= (ShftAmt)? SHFT_OUT[31:0] : B;	 
				MFHI: ALUOut <= hiloreg[63:32];
				MFLO: ALUOut <= hiloreg[31:0];
				default: ALUOut <= 0;
			endcase
	end
	assign sameSign = ((A[31]^~B[31]) && (ALUctl == ADD)) || ((A[31]^B[31]) && (ALUctl == SUB));
	assign signedOp = (ALUctl == ADD || ALUctl == SUB);
	assign diffOutSign = sameSign & (A[31]^ALUOut[31]);
	assign Ovfl = signedOp & diffOutSign;
	
	//synopsys translate_on
endmodule

module bin2unary ( out ,bin ,in, sign )  /* synthesis syn_black_box */  ;

input [4:0] bin ;
wire [4:0] bin ;
input wire sign;

output [31:0] out ;
wire [31:0] out ;
input [31:0] in ;
wire [31:0] in ;

reg [31:0] mask;
//}} End of automatically maintained section 

//synopsys translate_off

// -- Enter your statements here -- //
always @*
	case(bin) /* synthesis parallel_case */
		0: mask = 32'h00000001;
		1: mask = 32'h00000003;
		2: mask = 32'h00000007;
		3: mask = 32'h0000000F;
		4: mask = 32'h0000001F;
		5: mask = 32'h0000003F;
		6: mask = 32'h0000007F;
		7: mask = 32'h000000FF;
		8: mask = 32'h000001FF;
		9: mask = 32'h000003FF;
		10: mask = 32'h000007FF;
		11: mask = 32'h00000FFF;
		12: mask = 32'h00001FFF;
		13: mask = 32'h00003FFF;
		14: mask = 32'h00007FFF;
		15: mask = 32'h0000FFFF;
		16: mask = 32'h0001FFFF;
		17: mask = 32'h0003FFFF;
		18: mask = 32'h0007FFFF;
		19: mask = 32'h000FFFFF;
		20: mask = 32'h001FFFFF;
		21: mask = 32'h003FFFFF;
		22: mask = 32'h007FFFFF;
		23: mask = 32'h00FFFFFF;
		24: mask = 32'h01FFFFFF;
		25: mask = 32'h03FFFFFF;
		26: mask = 32'h07FFFFFF;
		27: mask = 32'h0FFFFFFF;
		28: mask = 32'h1FFFFFFF;
		29: mask = 32'h3FFFFFFF;
		30: mask = 32'h7FFFFFFF;
		31: mask = 32'hFFFFFFFF;	

endcase

assign out = in | ({32{sign}} & mask);

//synopsys translate_on
endmodule

module bit_reverse(in,out) /* synthesis syn_black_box */ ;
	input wire [31:0] in;
	output wire [31:0] out;
	
	//synopsys translate_off
	
	assign out = {in[0],in[1],in[2],in[3],in[4],in[5],in[6],in[7],in[8],in[9],in[10],in[11],in[12],in[13],in[14],in[15],in[16],in[17],in[18],in[19],in[20],in[21],in[22],in[23],in[24],in[25],in[26],in[27],in[28],in[29],in[30],in[31]};

	//synopsys translate_on
endmodule

module barrel32(DIN,DOUT,SHIFT) /* synthesis syn_black_box */;
    input [31:0] DIN;
    output [31:0] DOUT;
    input [4:0] SHIFT;
	
	//synopsys translate_off

wire [17:0] ENCODE;
wire [17:0] WORDA, WORDB, WORDC, WORDD;
wire [35:0] OUTA, OUTB, OUTC, OUTD;
wire [7:0] BYTE0,BYTE1,BYTE2,BYTE3;
reg [7:0] ONE_HOT;
//reg [35:0] DOUT_TEMP;

//assign DOUT = DOUT_TEMP;

assign ENCODE[17:8] = 10'b00000000;
assign ENCODE[7:0] = ONE_HOT;

assign WORDA[17:16] = 2'b00;
assign WORDB[17:16] = 2'b00;
assign WORDC[17:16] = 2'b00;
assign WORDD[17:16] = 2'b00;

assign WORDA[15:8] = DIN[7:0];
assign WORDB[15:8] = DIN[15:8];
assign WORDC[15:8] = DIN[23:16];
assign WORDD[15:8] = DIN[31:24];

assign WORDA[7:0] = 8'h00;
assign WORDB[7:0] = DIN[7:0];
assign WORDC[7:0] = DIN[15:8];
assign WORDD[7:0] = DIN[23:16];

always @(SHIFT[2:0])
begin
   case (SHIFT[2:0])
       3'b000 : ONE_HOT = 8'b00000001;   //0
       3'b001 : ONE_HOT = 8'b00000010;   //1
       3'b010 : ONE_HOT = 8'b00000100;   //2
       3'b011 : ONE_HOT = 8'b00001000;   //3
       3'b100 : ONE_HOT = 8'b00010000;   //4
       3'b101 : ONE_HOT = 8'b00100000;   //5
       3'b110 : ONE_HOT = 8'b01000000;   //6
       default: ONE_HOT = 8'b10000000;   //7
    endcase
end

`library("MULTA","ovi_unisim")
`library("MULTB","ovi_unisim")
`library("MULTC","ovi_unisim")
`library("MULTD","ovi_unisim")


MULT18X18 MULTA( .A(WORDA), .B(ENCODE), .P(OUTA));
MULT18X18 MULTB( .A(WORDB), .B(ENCODE), .P(OUTB));
MULT18X18 MULTC( .A(WORDC), .B(ENCODE), .P(OUTC));
MULT18X18 MULTD( .A(WORDD), .B(ENCODE), .P(OUTD));

assign BYTE0 = OUTA[15:8];
assign BYTE1 = OUTB[15:8];
assign BYTE2 = OUTC[15:8];
assign BYTE3 = OUTD[15:8];

assign DOUT[7:0] = (SHIFT[4:3] == 2'b00)? BYTE0 : 8'h00;
assign DOUT[15:8] = (SHIFT[4])? 8'h00 : ((SHIFT[3])? BYTE0 : BYTE1);
assign DOUT[23:16] = (SHIFT[4])? ((SHIFT[3])? 8'h00 : BYTE0) : ((SHIFT[3])? BYTE1 : BYTE2); 
assign DOUT[31:24] = (SHIFT[4])? ((SHIFT[3])? BYTE0 : BYTE1) : ((SHIFT[3])? BYTE2 : BYTE3); 

//synopsys translate_on

endmodule