`timescale 1ns/1ps

module ICache_4Word(			 
	input CLK, Reset,
	input [31:0] InstrAddr,
	output [31:0] Instruction,
	output	Stall, InvalidAddress,

	output CacheReadRequest,
	input CacheValid,
	input [127:0] CacheLine,
	output [27:0] CacheAddress,
	input CacheInvalidAddress) /* synthesis syn_black_box */ ;
	
	//synopsys translate_off

	wire CacheMiss;
	
	reg [27:0] Tag;
	reg [31:0] Line [3:0];
	reg	Valid;
	
	reg CacheReadRequest;
	
	//if the tag on the cache line doesn't match the address, we need to request more data from the memory
	assign CacheMiss = (InstrAddr[31:4]!=Tag);
	assign Stall = CacheMiss;	  
	assign Instruction = CacheMiss ? 32'b0 : Line[InstrAddr[3:2]];
	
	localparam NOREQUEST = 1,
		WAITFORVALID = 2;
	
	reg [2:0] state;
	
	assign CacheAddress = InstrAddr[31:4];
		
	always @(posedge CLK)
		if (Reset) begin
			state<=NOREQUEST;
			Tag<=28'hFFFF_FFF;
		end else
			case (state)
				NOREQUEST:
					if (CacheMiss) begin
						state<=WAITFORVALID;
						CacheReadRequest<=1'b1; //signal that we want to request a cache line from the memory
					end else begin //stay in this state
						state<=NOREQUEST;
						CacheReadRequest<=1'b0;
					end
				WAITFORVALID: begin
					CacheReadRequest<=1'b0;
					if (CacheValid) begin
						state<=NOREQUEST; //return to no request state		   
						Line[3]<=CacheLine[127:96];  //update cache data
						Line[2]<=CacheLine[95:64];
						Line[1]<=CacheLine[63:32];
						Line[0]<=CacheLine[31:0]; 
						Tag<=InstrAddr[31:4]; //update tag
					end else
						state<=WAITFORVALID; //stay in the same state
				end
			endcase				
			
	//synopsys translate_on
endmodule



module ICache_1Word(			 
	input CLK, Reset,
	input [31:0] InstrAddr,
	output [31:0] Instruction,
	output	Stall, InvalidAddress,

	output CacheReadRequest,
	input CacheValid,
	input [31:0] CacheLine,
	output [29:0] CacheAddress,
	input CacheInvalidAddress) /* synthesis syn_black_box */ ;
	
	//synopsys translate_off

	wire CacheMiss;
	
	reg [29:0] Tag;
	reg [31:0] Line;
	reg	Valid;
	
	reg CacheReadRequest;
	
	//if the tag on the cache line doesn't match the address, we need to request more data from the memory
	assign CacheMiss = (InstrAddr[31:2]!=Tag);
	assign Stall = CacheMiss;	  
	assign Instruction = CacheMiss ? 32'b0 : Line;
	
	localparam NOREQUEST = 1,
		WAITFORVALID = 2;
	
	reg [2:0] state;
	
	assign CacheAddress = InstrAddr[31:2];
		
	always @(posedge CLK)
		if (Reset) begin
			state<=NOREQUEST;
			Tag<=28'hFFFF_FFF;
		end else
			case (state)
				NOREQUEST:
					if (CacheMiss) begin
						state<=WAITFORVALID;
						CacheReadRequest<=1'b1; //signal that we want to request a cache line from the memory
					end else begin //stay in this state
						state<=NOREQUEST;
						CacheReadRequest<=1'b0;
					end
				WAITFORVALID: begin
					CacheReadRequest<=1'b0;
					if (CacheValid) begin
						state<=NOREQUEST; //return to no request state		   
						Line<=CacheLine; 
						Tag<=InstrAddr[31:2]; //update tag
					end else
						state<=WAITFORVALID; //stay in the same state
				end
			endcase				
	
	//synopsys translate_on
		
endmodule
